.container {
  position: relative;
  width: 100%;
  height: 100%;
  display: flex;

  .tags {
    flex-grow: 1;
    max-width: 175px;
  }

  .table {
    flex-grow: 1;
  }
}

.title {
  margin: 12px 0 25px 0;
  line-height: 22px;
  font-weight: 600;
}
