import { GraphColumnState } from 'modules/Graph/strategies/Grid/components/GraphColumn'

export const sleepRowColumnStatePaginated: Record<number, GraphColumnState[]> = {
  1: [
    {
      isVerticalLine: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        4
      ],
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false,
      isFirstRow: true
    },
    {
      isHorizontalLine: true,
      mergeSourceColumns: [
        4
      ],
      isVerticalLine: true,
      isColumnAboveEmpty: false,
      isFirstRow: true
    },
    {
      isHorizontalLine: true,
      mergeSourceColumns: [
        4
      ],
      isVerticalLine: true,
      isColumnAboveEmpty: false,
      isFirstRow: true
    },
    {
      isHorizontalLine: true,
      mergeSourceColumns: [
        4
      ],
      isFirstRow: true
    },
    {
      isLeftDownCurve: true,
      isFirstRow: true
    }
  ],
  2: [
    {
      isVerticalLine: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        3
      ],
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isHorizontalLine: true,
      mergeSourceColumns: [
        3
      ],
      isVerticalLine: true,
      isColumnAboveEmpty: false
    },
    {
      isHorizontalLine: true,
      mergeSourceColumns: [
        3
      ],
      isVerticalLine: true,
      isColumnAboveEmpty: false
    },
    {
      isLeftDownCurve: true
    },
    {
      isVerticalLine: true
    }
  ],
  3: [
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnAboveEmpty: false
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    }
  ],
  4: [
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    }
  ],
  5: [
    {
      isVerticalLine: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        1,
        2,
        2
      ],
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isLeftUpCurve: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        2,
        2
      ]
    },
    {
      isLeftUpCurve: true,
      isLeftDownCurve: true
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    }
  ],
  6: [
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    }
  ],
  7: [
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true
    }
  ],
  8: [
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    }
  ],
  9: [
    {
      isVerticalLine: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        2,
        3,
        4,
        3
      ],
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        2,
        3,
        4,
        3
      ]
    },
    {
      isLeftUpCurve: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        3,
        4,
        3
      ]
    },
    {
      isLeftUpCurve: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        4
      ],
      isLeftDownCurve: true
    },
    {
      isLeftUpCurve: true
    }
  ],
  10: [
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {},
    {
      isVerticalLine: true
    },
    {}
  ],
  11: [
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {},
    {
      isVerticalLine: true
    },
    {}
  ],
  12: [
    {
      isVerticalLine: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        1,
        2
      ],
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isLeftUpCurve: true,
      isHorizontalLine: true,
      mergeSourceColumns: [
        2
      ]
    },
    {
      isLeftDownCurve: true
    },
    {
      isVerticalLine: true
    },
    {}
  ],
  13: [
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true
    },
    {}
  ],
  14: [
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {}
  ],
  15: [
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {}
  ],
  16: [
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {}
  ],
  17: [
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {}
  ],
  18: [
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {}
  ],
  19: [
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false
    },
    {}
  ],
  20: [
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false,
      isLastRow: true
    },
    {
      isVerticalLine: true,
      isNode: true,
      isColumnAboveEmpty: false,
      isColumnBelowEmpty: false,
      isLastRow: true
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false,
      isLastRow: true
    },
    {
      isVerticalLine: true,
      isColumnBelowEmpty: false,
      isLastRow: true
    },
    {
      isLastRow: true
    }
  ]
}