# Generated by Django 6.0.1 on 2026-01-29 18:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0006_add_message_count_and_kind_index'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModelPrice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('model_id', models.CharField(max_length=100)),
                ('effective_date', models.DateField()),
                ('input_price', models.DecimalField(decimal_places=6, max_digits=12)),
                ('output_price', models.DecimalField(decimal_places=6, max_digits=12)),
                ('cache_read_price', models.DecimalField(decimal_places=6, max_digits=12)),
                ('cache_write_5m_price', models.DecimalField(decimal_places=6, max_digits=12)),
                ('cache_write_1h_price', models.DecimalField(decimal_places=6, max_digits=12)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'indexes': [models.Index(fields=['model_id', '-effective_date'], name='core_modelp_model_i_8128b9_idx')],
                'unique_together': {('model_id', 'effective_date')},
            },
        ),
    ]
