# Generated by Django 6.0.1 on 2026-01-29 15:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0004_sessionitemlink'),
    ]

    operations = [
        migrations.AddField(
            model_name='session',
            name='title',
            field=models.CharField(blank=True, max_length=250, null=True),
        ),
    ]
