.container {
  position: relative;
}

.graph {
  height: 100%;
  display: grid;
  gap: 0;
}

.dragHandle {
  position: absolute;
  right: -5px;
  top: 10px;
  width: 10px;
  height: 100%;
  z-index: 100;
  opacity: 0;
  transition: all ease-in-out 0.2s;

  &:hover {
    opacity: 1;
    cursor: ew-resize;
    border-left: none;
    border-right: 1px solid #cccccc;
    border-bottom: none;
    border-top: none;
    background: none;
  }
}