.commitNode {
  all: unset;
  border-radius: 50%;
  z-index: 20;
  position: relative;

  &:hover {
    cursor: pointer;
  }

  .commitLabel {
    position: absolute;
    padding: 2px 5px 2px 2px;
    border-radius: 5px;
    font-size: 0.7rem
  }
}

.mergeCommitInner {
  border-radius: 50%;
  position: absolute;
}