/**
 * The number of pixels that the header
 * row needs to be so that the first
 * data row lines up with the first row
 * on the graph. Assuming 0 row spacing.
 */
export const HEADER_ROW_HEIGHT = 47

/**
 * The number of pixels of margin at the
 * top of the table element that offsets
 * it enough to line-up correctly with
 * the graph
 */
export const TABLE_MARGIN_TOP = 12