# Generated by Django 6.0.1 on 2026-02-01 21:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0018_add_project_total_cost'),
    ]

    operations = [
        migrations.AddField(
            model_name='sessionitem',
            name='timestamp',
            field=models.DateTimeField(blank=True, db_index=True, null=True),
        ),
    ]
