# Generated by Django 6.0.2 on 2026-02-27 00:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0038_recompute_project_sessions_count_with_created_at'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='session',
            name='idx_session_by_date',
        ),
        migrations.AddField(
            model_name='session',
            name='permission_mode',
            field=models.CharField(default=None, max_length=30, null=True),
        ),
        migrations.AddIndex(
            model_name='session',
            index=models.Index(fields=['project', 'type'], name='idx_session_project_type'),
        ),
    ]
