# Generated by Django 6.0.1 on 2026-01-24 17:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='session',
            name='compute_version',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sessionitem',
            name='display_level',
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sessionitem',
            name='group_head',
            field=models.PositiveIntegerField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name='sessionitem',
            name='group_tail',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
