.tagContainer {
  all: unset;
  position: relative;
  display: flex;
  align-items: center;
  justify-content: flex-end;
  width: 100%;

  .tag {
    display: flex;
    align-items: center;
    justify-content: flex-end;
    padding: 2px 6px;
    border-radius: 6px;
    max-width: 140px;
    font-size: 0.8rem;
    height: 22px;

    opacity: 0.8;
    transition: all ease-in-out 0.2s;

    &:hover {
      opacity: 1;
    }
  }

  .tagLine {
    $size: 1px;
    top: 50%;
    height: $size;
    position: absolute;
    transition: opacity ease-in-out 0.3s;
  }
}