.session-toast-content[data-v-b69ce41b]{display:flex;flex-direction:column;gap:var(--wa-space-xs);margin-top:var(--wa-space-xs)}.session-toast-row[data-v-b69ce41b]{display:flex;align-items:center;gap:var(--wa-space-s);min-width:0}.session-toast-label[data-v-b69ce41b]{opacity:.8;flex-shrink:0;width:3rem}.session-toast-project[data-v-b69ce41b]{font-weight:700;min-width:0}.session-toast-session[data-v-b69ce41b]{display:flex;gap:var(--wa-space-s);margin-bottom:var(--wa-space-xs)}.session-toast-session .session-toast-label[data-v-b69ce41b]{align-self:flex-start}.session-toast-title[data-v-b69ce41b]{font-weight:700}.session-toast-error[data-v-b69ce41b]{color:var(--wa-color-danger);font-weight:700}.session-toast-content>wa-button[data-v-b69ce41b]{align-self:flex-end}
