/**
 * The height of each row across
 * all subcomponents, in pixels.
 */
export const ROW_HEIGHT = 40

/**
 * The default width and height of
 * a commit node on the graph.
 */
export const DEFAULT_NODE_SIZE = 20

/**
 * The width of the borders on a commit node.
 */
export const NODE_BORDER_WIDTH = 2

/**
 * The number of pixels to offset the background
 * element/colour by so that it has padding around
 * the commit node on the graph.
 */
export const BACKGROUND_HEIGHT_OFFSET = 16

/**
 * The size of the SVG curved line and
 * the lengths of the accompanying vertical
 * lines.
 */
export const CURVE_SIZE = 24