# Generated by Django 6.0.1 on 2026-01-30 05:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0012_remove_sessionitemlink_idx_item_link_lookup_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='session',
            name='cwd',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
        migrations.AddField(
            model_name='session',
            name='git_branch',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='session',
            name='model',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]
