# Generated by Django 6.0.2 on 2026-02-23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0031_add_activity_cost'),
    ]

    operations = [
        migrations.RenameField(
            model_name='weeklyactivity',
            old_name='count',
            new_name='user_message_count',
        ),
        migrations.RenameField(
            model_name='dailyactivity',
            old_name='count',
            new_name='user_message_count',
        ),
    ]
