.indexNode {
  border-radius: 50%;
  z-index: 20;
}

@keyframes spin {
  from {
    transform: rotate(0deg);
  }
  to {
    transform: rotate(360deg);
  }
}

.spin {
  animation: spin 10s linear infinite;
}