# Generated by Django 6.0.2 on 2026-02-11 22:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0025_split_session_item_links'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='git_root',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
    ]
