# Generated by Django 6.0.1 on 2026-01-28 18:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0003_add_kind_to_sessionitem'),
    ]

    operations = [
        migrations.CreateModel(
            name='SessionItemLink',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('source_line_num', models.PositiveIntegerField()),
                ('target_line_num', models.PositiveIntegerField()),
                ('link_type', models.CharField(max_length=50)),
                ('reference', models.CharField(max_length=255)),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='item_links', to='core.session')),
            ],
            options={
                'indexes': [models.Index(fields=['session', 'source_line_num', 'link_type', 'reference'], name='idx_item_link_lookup')],
            },
        ),
    ]
