# Generated by Django 6.0.2 on 2026-02-06 01:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0021_session_archived'),
    ]

    operations = [
        migrations.RenameField(
            model_name='session',
            old_name='git_branch',
            new_name='cwd_git_branch',
        ),
        migrations.AddField(
            model_name='session',
            name='git_directory',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
        migrations.AddField(
            model_name='session',
            name='git_branch',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='sessionitem',
            name='git_directory',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
        migrations.AddField(
            model_name='sessionitem',
            name='git_branch',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
