/**
 * The number of pixels of margin at the
 * top of the graph element that offsets
 * it enough to line-up correctly with
 * adjacent elements (such as tags or a table).
 */
export const GRAPH_MARGIN_TOP = 12