# Generated by Django 6.0.1 on 2026-01-29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0007_add_modelprice'),
    ]

    operations = [
        # Add cost and context usage fields to Session
        migrations.AddField(
            model_name='session',
            name='context_usage',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='session',
            name='total_cost',
            field=models.DecimalField(blank=True, decimal_places=6, max_digits=10, null=True),
        ),
        # Add cost and context usage fields to SessionItem
        migrations.AddField(
            model_name='sessionitem',
            name='message_id',
            field=models.CharField(blank=True, db_index=True, max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='sessionitem',
            name='cost',
            field=models.DecimalField(blank=True, decimal_places=6, max_digits=10, null=True),
        ),
        migrations.AddField(
            model_name='sessionitem',
            name='context_usage',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
