# Generated by Django 6.0.1 on 2026-01-30 01:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0010_add_self_cost_and_subagents_cost'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sessionitemlink',
            name='target_line_num',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
