# Generated by Django 6.0.1 on 2026-01-30 02:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0011_alter_sessionitemlink_target_line_num'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='sessionitemlink',
            name='idx_item_link_lookup',
        ),
        migrations.AddIndex(
            model_name='sessionitemlink',
            index=models.Index(fields=['session', 'link_type', 'reference', 'source_line_num'], name='idx_item_link_lookup'),
        ),
    ]
