<template>
  <svg
    class="icon"
    viewBox="0 0 24 24"
    xmlns="http://www.w3.org/2000/svg"
  >
    <path d="M20.59 13.41l-7.17 7.17a2 2 0 0 1-2.83 0L2 12V2h10l8.59 8.59a2 2 0 0 1 0 2.82z" />
    <line x1="7" y1="7" x2="7.01" y2="7" />
  </svg>
</template>

<style scoped>
.icon {
  width: 1rem;
  margin-top: 0.125rem;
  flex: 0 0 auto;
  margin-left: 0.25rem;
  stroke-width: 2px;
  stroke-linecap: round;
  stroke-linejoin: round;
  stroke: currentColor;
  fill: none;
}
</style>
