# Generated by Django 6.0.2 on 2026-02-27 01:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0039_session_permission_mode'),
    ]

    operations = [
        migrations.AddField(
            model_name='session',
            name='selected_model',
            field=models.CharField(default=None, max_length=30, null=True),
        ),
    ]
