# Generated by Django 6.0.1 on 2026-02-06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0022_session_rename_git_branch_to_cwd_git_branch'),
    ]

    operations = [
        migrations.AddField(
            model_name='session',
            name='pinned',
            field=models.BooleanField(default=False),
        ),
    ]
