# Generated by Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0019_add_session_item_timestamp'),
    ]

    operations = [
        migrations.RenameField(
            model_name='project',
            old_name='archived',
            new_name='stale',
        ),
        migrations.RenameField(
            model_name='session',
            old_name='archived',
            new_name='stale',
        ),
    ]
