# Generated by Django 6.0.1 on 2026-01-31 18:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0015_add_session_project_mtime_index'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='color',
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name='project',
            name='name',
            field=models.CharField(blank=True, max_length=25, null=True),
        ),
    ]
